<?php

/**
 * @file
 * Webform module tabindex attribute handling.
 */

/**
 * Provides interface and database handling for reset/update
 * the position of tabindex in webform elements.
 */

/**
 * The table-based listing of all components for this webform in tabindex order.
 */
function webform_tabindex_form($form_state, $node) {

  $form = array(
    '#tree' => TRUE,
    '#node' => $node,
    'components' => array(),
  );

  $form['nid'] = array(
    '#type' => 'value',
    '#value' => $node->nid,
  );

  $result = db_query("SELECT wc.nid, wc.cid, wc.name, wc.type, wc.form_key, wt.tabindex FROM {webform_component} AS wc LEFT JOIN {webform_tabindex} AS wt ON wc.cid = wt.cid WHERE wc.nid = %d GROUP BY wc.cid ORDER BY wt.tabindex ASC", $node->nid);
    // Use to set weight of tabindex if corresponding record does not exists
    // into webform_tabindex module table.
    $counter = 1;

    // Iterate through each database result.
    while ($item = db_fetch_object($result)) {
      // Create a form entry for this item.
      // Each entry will be an array using the the unique id for that item as
      // the array key, and an array of table row data as the value.
      $form['tabindex_items'][$item->cid] = array(
        // We'll use a form element of type '#markup' to display the item name.
        'name' => array(
          '#markup' => check_plain($item->name),
        ),
        // We'll use a form element of type '#markup' to display the item type.
        'type' => array(
          '#markup' => check_plain($item->type),
        ),
        // We'll use a form element of type '#markup' to display the item
        // form_key.
        'form_key' => array(
          '#markup' => check_plain($item->form_key),
        ),
        'operation' => array(
          '#markup' => l(t('Edit'), 'node/' . $node->nid . '/webform/components/' . $item->cid, array('query' => drupal_get_destination())),
        ),
        // The 'weight' field will be manipulated as we move the items around in
        // the table using the tabledrag activity.  We use the 'weight' element
        // defined in Drupal's Form API.
        'weight' => array(
          '#type' => 'textfield',
          '#title' => '',
          '#default_value' => isset($item->tabindex) ? $item->tabindex : $counter,
          '#maxlength' => 3,
          '#title-display' => 'invisible',
          '#size' => 4,
          '#attributes' => array('class' => 'weight'),
        ),
      );
      $counter++;
    }

  // Now we add our submit button, for submitting the form results.
  // The 'actions' wrapper used here isn't strictly necessary for tabledrag,
  // but is included as a Form API recommended practice.
  $form['submit'] = array(
    '#type' => 'submit',
    '#value' => t('Save changes'),
    '#validate' => array('webform_tabindex_form_validate'),
    '#submit' => array('webform_tabindex_from_submit'),
  );
  return $form;
}

/**
 * Validate handler for webform_tabindex_form().
 */
function webform_tabindex_form_validate($form, &$form_state) {
  $tabindex_items = $form_state['values']['tabindex_items'];
  if (is_array($tabindex_items) && count($tabindex_items)) {
    foreach ($tabindex_items as $cid => $item) {
      if (trim($item['weight']) == '' || $item['weight'] <= 0) {
        form_error($form['components'], t('!name: Tabindex number should be a positive integer value.', array('!name' => $form['tabindex_items'][$cid]['name']['#markup'])) . theme('item_list', array('tabindex_items' => $item))
        );
      }
    }
  }
}

/**
 * Theme the node components form. Use a table to organize the tabindex.
 *
 * @return array
 *   Formatted HTML form, ready for display.
 */
function theme_webform_tabindex_form($form) {
  $node = $form['#node'];
  
  // the variable that will hold our form HTML output
  $output = '';

  // Initialize the variable which will store our table rows.
  $rows = array();
  // Iterate over each element in our $form['tabindex_items'] array.
  foreach ($form['tabindex_items'] as $cid => $row ) {
    if (!intval($cid)) {
      continue;
    }
    // This array will hold the table cells for a row.
    $this_row = array();

    // first, add the "name" markup
    $this_row[] = $form['tabindex_items'][$cid]['name']['#markup'];
    // Add forms element 'type' column.
    $this_row[] = $form['tabindex_items'][$cid]['type']['#markup'];
    // Add forms element 'form_key' column.
    $this_row[] = $form['tabindex_items'][$cid]['form_key']['#markup'];
    // Add forms element 'weight' column.
    $this_row[] = drupal_render($form['tabindex_items'][$cid]['weight']);
    // Add forms element 'operation' column.
    $this_row[] = $form['tabindex_items'][$cid]['operation']['#markup'];

    //Add the row to the array of rows
    $table_rows[] = array('data' => $this_row, 'class'=>'draggable');
  }

  // We now define the table header values.  Ensure that the 'header' count
  // matches the final column count for your table.
  $header = array(t('Name'),
    t('Type'),
    t('Form  Key'),
    t('Tabindex Number'),
    t('Operations'),
  );

  // We also need to pass the drupal_add_tabledrag() function an id which will
  // be used to identify the <table> element containing our tabledrag form.
  // Because an element's 'id' should be unique on a page, make sure the value
  // you select is NOT the same as the form ID used in your form declaration.
  $table_id = 'webform-tabindex-table';

  // this function is what brings in the javascript to make our table drag-and-droppable
  drupal_add_tabledrag($table_id, 'order', 'sibling', 'weight'); 

  // We can render our tabledrag table for output.
  $form['tabindex_items'] = array(
    '#type' => 'markup',
    '#value' => theme('table', $header, $table_rows, array('id' => $table_id)),
    '#weight' => '1',
    );
  $output = drupal_render($form);
  return $output;
}

/**
 * Submit handler for webform_tabindex_form() to save tab/navigation order.
 */
function webform_tabindex_from_submit($form, &$form_state) {
  $nid = $form_state['values']['nid'];
  // Iterate over each element in our $form_state['values']['tabindex_items']
  // array.
  foreach ($form_state['values']['tabindex_items'] as $cid => $item) {
    //drupal_set_message($cid);
    $query = "SELECT count(*) AS count FROM {webform_tabindex} WHERE nid = %d AND cid = %d";
  
    $count = db_result(db_query($query, $nid, $cid));
    if ($count) {
      db_query("UPDATE {webform_tabindex} SET tabindex = %d WHERE nid = %d AND cid = %d", $item['weight'], $nid, $cid);
    }
    else {
      db_query("INSERT INTO {webform_tabindex} (nid, cid, tabindex) VALUES (%d, %d, %d)", $nid, $cid, $item['weight']);
    }
  }
  drupal_set_message(t('The Tab Index positions have been updated.'));
}
